% poisson_process_test.m
% ==========
% tests occurrence times (t) to see if the reposes are consistent with an exponential distribution

global r rstar v % reposes, incomplete last repose, volume

init_volcano; % reads in data and sets it up

% fit Exponential renewal process
% f(r) = nu*exp(-nu*r)
nu = n/(T - S)
LL = n*log(nu) - n
AIC = -2 + 2*LL

% test fit to Exponential distribution
% calculate Kaplan-Meier Product-Limit estimate of survival function
Sn = PLestimate([r; rstar]);
rn = sort([0; r; r; rmax]); % step times of empirical survival function
En = exp(-nu*rn); % fitted Exponential distribution 
% Kolmogorov-Smirnov 2-sided 5% critical value
KSc = kolmogorov_smirnov2(n-1); %dof = n renewals, less 1 parameter estimated from data
Dn = max(abs(Sn - En)); % test statistic
if Dn < KSc
    Expres = 'Exponential';
else
    Expres = 'non-Exponentiall';
end

% output
Eplot = exp(-nu*rplot);
figure(3)
plot(rn,Sn,'-k',rplot,Eplot,'--k',rplot,Eplot+KSc,':k'); 
hold on;
legend('Observed','Exponential renewals','95% Confidence band');
plot(rplot,Eplot-KSc,':k'); 
hold off;
xlabel('t (years)'); 
ylabel('S(t)'); 
ylim([0 1]);
xlim([0 rmax]);
text(0.6*rmax,0.5,['D_n=' num2str(Dn) ', ' Expres]);


