function simulation = simulate_hp(mu, T)
% Inputs
%   - mu -> Numerical field. Intensity of the process.
%   - T -> The total time of the process. Process over the interval [0,T]
% Outputs
%   - simulation -> Array with the simulated event history
%
% Simulation of Poisson processes on the interval [0,T] with given CIF.
% The intensity function for this model is
% lambda(t) = mu
%
% Reference:
% - P. Laub - DOI: 10.1007/978-3-030-84639-8
% The elements of Hawkes processes (Section 2.2)
    
    interarrival_times = exprnd(1 / mu, 1, ceil(1.2 * T * mu) + 1); % Generate interarrival times
    simulation = cumsum(interarrival_times); % Get event times from interarrival times
        
    % If the end of the interval was not reached, generate more
    % times and append to the simulated data
    while simulation(end) < T 
        interarrival_times = exprnd(1 / mu, 1, ceil(0.2 * T * mu));
        simulation = [simulation (simulation(end) + cumsum(interarrival_times))]; 
    end
    simulation = simulation(simulation <= T);
end
 