% This file shows that the bootstrap procedure is needed for getting consistent
% p-values from a test for point processes.
% This test was taken from the file `poisson_process_test.m`, provided in
% Bebbington, Mark S. (2013) "Models for Temporal Volcanic Hazard," Statistics in Volcanology. DOI 10.5038/2163-338X.1.1
% Here, we perform the test 10000 times, each time in a different process generated by
% a Poisson process (file `simulate_hp` simulates the processes) over the time interval
% [0, 10000] with parameters mu = 0.1 (expected number of events is 1000).
% The variable `p5s` count how many times the test returned a p-value smaller then 5%. If
% the test is consistent, then this should happen in approximately 5% of the times.
% In the end, the percentage of times the test returned a p-value smaller than 5% is printed.
% This test usually fails in about 0.5% of the times.

nruns = 10000;
T = 10000;
mu = 0.1;
p5s = 0;
for i = 1:nruns
    sim = simulate_hp(mu, T);
    sim = sim(:);
    r = diff([sim; T]);
    rmax = 1.1*max(r);
    rstar = sim(1);
    n = length(sim);
    nu = n / T;
    Sn = PLestimate([r; rstar]);
    rn = sort([0; r; r; rmax]); % step times of empirical survival function
    En = exp(-nu*rn); % fitted Exponential distribution 
    % Kolmogorov-Smirnov 2-sided 5% critical value
    KSc = kolmogorov_smirnov2(n-1); %dof = n renewals, less 1 parameter estimated from data
    Dn = max(abs(Sn - En)); % test statistic
    p5s = p5s + (Dn >= KSc);
end

fprintf("Percentage of times the test failed: %.2f%%\n", 100 * p5s / nruns)